(********************************************************************************)
(*                        UNIFIED INTERBASE (UIB)                               *)
(*                                                                              *)
(* The contents of this file are subject to the Mozilla Public License Version  *)
(* 1.1 (the "License"); you may not use this file except in compliance with the *)
(* License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ *)
(*                                                                              *)
(* Software distributed under the License is distributed on an "AS IS" basis,   *)
(* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for *)
(* the specific language governing rights and limitations under the License.    *)
(*                                                                              *)
(* Unit owner : Henri Gourvest <hgourvest@progdigy.com>                         *)
(*                                                                              *)
(********************************************************************************)

//=== ct9999 Modify for CodeTyphon Studio =====================

unit uibsqledit;

{$I uib.inc}

interface
uses

  LCLIntf, LCLType, LMessages,
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, uib, uibmetadata, uibsynedit, ExtCtrls, ImgList, SynEdit, ToolWin,
  Grids, StdCtrls, uiblib, SynEditTypes, SynHighlighterSQL, uibkeywords;

type

  { TUIBSQLEditForm }

  TUIBSQLEditForm = class(TForm)
    Editor: TSynEdit;
    DDLEditor: TSynEdit;
    SQLHighLighter: TSynSQLSyn;
    ToolBar: TToolBar;
    RunBt: TToolButton;
    BtFetch: TToolButton;
    BtCommit: TToolButton;
    BtRollBack: TToolButton;
    ButtonImages: TImageList;
    ToolButton1: TToolButton;
    btRefresh: TToolButton;
    PageControl: TPageControl;
    SQLTab: TTabSheet;
    TabDDL: TTabSheet;
    Splitter: TSplitter;
    TreeView: TTreeView;
    ResultPage: TTabSheet;
    ResultGrid: TStringGrid;
    Splitter1: TSplitter;
    Log: TSynEdit;
    Panel2: TPanel;
    BtOK: TButton;
    BtCancel: TButton;
    ImageList: TImageList;
    procedure RunBtClick(Sender: TObject);
    procedure BtFetchClick(Sender: TObject);
    procedure BtCommitClick(Sender: TObject);
    procedure BtRollBackClick(Sender: TObject);
    procedure BtOKClick(Sender: TObject);
    procedure EditorKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure btRefreshClick(Sender: TObject);
    function Run: TUIBStatementType;
    procedure AddSQLTokens;
  private
    FStatement: TUIBStatement;
    HaveConnection: boolean;
    InsertList, ItemList: TStringList;
    Query: TUIBQuery;
    procedure ShowNodes(node: TMetaNode; from: TTreeNode);
    procedure SetStatement(const Value: TUIBStatement);
    procedure CheckTransaction;
  public
    property Statement: TUIBStatement Read FStatement write SetStatement;
  end;

var
  UIBSQLEditForm: TUIBSQLEditForm;

implementation

{$R *.frm}

{ TUIBSQLEditForm }

procedure TUIBSQLEditForm.FormCreate(Sender: TObject);

  procedure FontChange;
  var
    ChangeFont: TFont;
    i: integer;
  begin
    //The default font of its own country is called.
    ChangeFont := TFont.Create;
    try
      //In Japanese, since the size of a default font was small,
      // size was changed.
      ChangeFont.Size := 10;
      for i := 0 to ComponentCount-1 do
      begin
       if Components[i] is TSynEdit then
         TSynEdit(Components[i]).Font.Assign(ChangeFont) else
         if Components[i] is TStringGrid then
           TStringGrid(Components[i]).Font.Assign(ChangeFont);
      end;
    finally
      ChangeFont.Free;
    end;
  end;

begin

  HaveConnection := True;
  InsertList := TStringList.Create;
  ItemList := TStringList.Create;
  Query := TUIBQuery.Create(nil);

end;

procedure TUIBSQLEditForm.SetStatement(const Value: TUIBStatement);

//---------------------------------
procedure DisableEditors;
begin
  HaveConnection := False;
  ToolBar.Enabled := False;
  TabDDL.TabVisible := False;
end;
//---------------------------------
begin
  FStatement := Value;
  Query.Transaction := FStatement.Transaction;
  Editor.Lines.Assign(FStatement.SQL);
  try
    if (Statement.Transaction <> nil) and (Statement.Transaction.DataBase <> nil) then
      try
        // Metadata
        SQLHighLighter.BeginUpdate;
        ShowNodes(TMetaNode(Statement.Transaction.DataBase.GetMetaData), nil);
        SQLHighLighter.EndUpdate;
      except
        DisableEditors;
      end else
        DisableEditors;
  finally
    AddSQLTokens;
  end;
end;

procedure TUIBSQLEditForm.ShowNodes(node: TMetaNode; from: TTreeNode);
var
  i, j: Integer;
  ClassNode: TTreeNode;
begin
  if not (Node is TMetaDatabase) then
  begin
    from := TreeView.Items.AddChild(from, node.Name);
    from.Data := Node;
    if (node.NodeType = MetaTableField) then
    with TMetaTableField(node) do
    begin
      if (FieldInfos >= [fiprimary, fiforeign]) then from.ImageIndex := 13 else
      if (FieldInfos >= [fiprimary]) then from.ImageIndex := NodeInfos[MetaPrimary].icon else
      if (FieldInfos >= [fiforeign]) then from.ImageIndex := NodeInfos[MetaForeign].icon else
        from.ImageIndex := NodeInfos[node.NodeType].icon;
    end else
      from.ImageIndex := NodeInfos[node.NodeType].icon;
    from.SelectedIndex := from.ImageIndex;

    if node.NodeType in [MetaException, MetaGenerator, MetaUDF, MetaView,
      MetaProcedure, MetaRole, MetaTable, MetaDomain, MetaTrigger] then
    begin
      InsertList.Add(node.Name);
      if (node.NodeType = MetaDomain) then
      ItemList.Add(format('\image{%d} \color{$%x} %s \color{clBlack} \style{+B}%s\style{-B} %s',
        [NodeInfos[node.NodeType].Icon, NodeInfos[node.NodeType].color, node.NodeClass, Node.Name,
         TMetaDomain(node).ShortFieldType])) else
      ItemList.Add(format('\image{%d} \color{$%x} %s \color{clBlack} \style{+B}%s\style{-B}%s',
        [NodeInfos[node.NodeType].Icon, NodeInfos[node.NodeType].color, node.NodeClass,
         Node.Name, GetParams(node)]));

    end;
  end;

  for i := 0 to node.NodeCount - 1 do
    if node.Nodes[i].Childs.Count > 0 then
    begin
      ClassNode := TreeView.Items.AddChild(from, node.Nodes[i].ClassID.NodeClass + 's');
      ClassNode.ImageIndex := NodeInfos[node.Nodes[i].ClassID.NodeType].icon;
      ClassNode.SelectedIndex := ClassNode.ImageIndex;
      for j := 0 to node.Nodes[i].Childs.Count - 1 do
        ShowNodes(TMetaNode(node.Nodes[i].Childs[j]), ClassNode);
    end;
end;

procedure TUIBSQLEditForm.RunBtClick(Sender: TObject);
begin
  Run;
  ResultPage.TabVisible := False;
  PageControl.ActivePage := SQLTab;
  CheckTransaction;
end;

procedure TUIBSQLEditForm.BtFetchClick(Sender: TObject);
var
  Fetchime: Cardinal;
  i: Integer;
  st: TUIBStatementType;
  procedure Display(n: Integer);
  var j: Integer;
  begin
    ResultGrid.Cells[0, n] := Inttostr(n);
    for j := 1 to Query.Fields.FieldCount do
    begin
      if (Query.Fields.FieldType[j-1] = uftBlob) then ResultGrid.Cells[j, n] := '[blob]' else
      if Query.Fields.IsNull[j-1] then ResultGrid.Cells[j, n] := 'null' else
      ResultGrid.Cells[j, n] := Query.Fields.AsString[j-1];
    end;
  end;
begin
  st := Run;
  if (not (st in [stSelect, stExecProcedure]))
    then exit;

  if (st = stSelect) then
  begin                     
    Fetchime := GetTickCount64;
    Query.FetchAll;
    if (Query.Fields.RecordCount = 0) then Exit;
    Log.Lines.Add(format('Fetch: %d ms.', [GetTickCount64 - Fetchime]));
  end;

  if (st = stExecProcedure) and (Query.Fields.FieldCount = 0) then
    Exit;

  ResultGrid.ColCount := Query.Fields.FieldCount + 1;
  if (st = stExecProcedure) then
    ResultGrid.RowCount := 2 else
    ResultGrid.RowCount := Query.Fields.RecordCount + 1;

  for i := 1 to Query.Fields.FieldCount do
    ResultGrid.Cells[i, 0] := Query.Fields.AliasName[i-1];

  if (st = stExecProcedure) then
    Display(1) else
  for i := 1 to Query.Fields.RecordCount do
  begin
    Query.Fields.CurrentRecord := i - 1;
    Display(i);
  end;

  ResultPage.TabVisible := True;
  PageControl.ActivePage := ResultPage;
  CheckTransaction;
end;

procedure TUIBSQLEditForm.BtCommitClick(Sender: TObject);
begin
  Query.Transaction.Commit;
  CheckTransaction;
end;

procedure TUIBSQLEditForm.BtRollBackClick(Sender: TObject);
begin
  Query.Transaction.RollBack;
  CheckTransaction;
end;

procedure TUIBSQLEditForm.BtOKClick(Sender: TObject);
begin
  Statement.SQL.Assign(Editor.Lines);
end;

procedure TUIBSQLEditForm.CheckTransaction;
begin
  BtCommit.Enabled := Statement.Transaction.InTransaction;
  BtRollBack.Enabled := BtCommit.Enabled;
end;

procedure TUIBSQLEditForm.EditorKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssShift]) and (Key = 120) then BtFetch.Click else
  if (Shift = []) and (Key = 120) then RunBt.Click;
end;

procedure TUIBSQLEditForm.FormDestroy(Sender: TObject);
begin
  InsertList.Free;
  ItemList.Free;
  Query.Free;
end;

procedure TUIBSQLEditForm.TreeViewChange(Sender: TObject; Node: TTreeNode);
begin
  if (node.Data <> nil) then
    DDLEditor.Text := TMetaNode(node.Data).AsDDL else
    DDLEditor.Lines.Clear;
end;

procedure TUIBSQLEditForm.btRefreshClick(Sender: TObject);
begin
  TreeView.Items.Clear;
  ItemList.Clear;
  InsertList.Clear;
  DDLEditor.Lines.Clear;
  SQLHighLighter.BeginUpdate;
  ShowNodes(TMetaNode(Statement.Transaction.Database.GetMetadata(True)), nil);
  SQLHighLighter.EndUpdate;
  AddSQLTokens;
end;

function TUIBSQLEditForm.Run: TUIBStatementType;
var
  PrepTime, ExecTime: cardinal;
begin
  result := TUIBStatementType(-1);
  Log.Lines.Clear;
  try
    Query.SQL.Assign(Editor.Lines);
    Query.Transaction.StartTransaction;
    PrepTime := GetTickCount64;
    query.Prepare;
    PrepTime := GetTickCount64 - PrepTime;
    ExecTime := GetTickCount64;
    Query.Execute;
    ExecTime := GetTickCount64 - ExecTime;
  except
    on E: EUIBError do
      begin
        log.Highlighter := nil;
        log.Lines.Text := E.Message;
        CheckTransaction;
        exit;
      end;
    else
      raise;
  end;

  Log.Highlighter := SQLHighLighter;
  Result := Query.StatementType;
  if Result = stSelect then
    Log.Lines.Add(Query.Plan);
  Log.Lines.Add(format('Prepare: %d ms.', [PrepTime]));
  Log.Lines.Add(format('Execute: %d ms.', [ExecTime]));
end;

procedure TUIBSQLEditForm.AddSQLTokens;
var
  i: Integer;
  c: TCharacterSet;
begin
  for i := low(SQLToKens) to high(SQLToKens) do
  begin
    InsertList.Add(SQLToKens[i]);
    ItemList.Add(format(SQLITEMSTR, [SQLToKens[i]]));
  end;
  for c := low(TCharacterSet) to high(TCharacterSet) do
  begin
    InsertList.Add(CharacterSetStr[c]);
    ItemList.Add(format(CHARSETITEMSTR, [CharacterSetStr[c]]));
  end;
end;

end.



